#ifndef _ARCEQM_H_
#ifndef _EVENTTOOLS_H_

#ifdef MAKE_EVNTOOLS_DLL
#define EVNTOOLS_EXPORT __declspec(dllexport) 
#elif EVNTOOLS_DLL
#define EVNTOOLS_EXPORT __declspec(dllimport) 
#else
#define EVNTOOLS_EXPORT 
#endif

#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

typedef struct
{
  short numProperties;
  short numDevices;
  long  eventCode;
  char  comment[64];
} AIB;  /* Archive Info Block */
#define AIBStructSize (2*2+4+64)

typedef struct
{
  float SamplingRate;
  long  numPreTriggerValues;
  long  TimeOffset;
  long  LowCount;
  long  HighCount;
  char  Units[16];
  double LowEGU;
  double HighEGU;
  short  numDevices;
  short  reserved;
} PIB;  /* Property Info Block */
#define PIBStructSize (5*4+16+2*8+2*2)

typedef struct
{
   char channelID[32]; /* Kanalidentifikator                         */
   UINT32 eventcode;   /* Ereignisbewerter                           */
   UINT16 arraysize;   /* Anzahl der Daten                           */
   UINT16 datatype;    /* Datentyp                                   */
   float samplingrate; /* Abtastfrequenz                             */
   UINT32 pretrigger;  /* Anzahl der Daten vor dem Triggerzeitpunkt  */
   UINT32 timeoffset;  /* Zeitoffset                                 */
   UINT32 lowcount;    /* kleinster Mewert in digits                */
   UINT32 highcount;   /* grter Mewert in digits                  */
   char egu[16];       /* physikalische Einheit                      */
   float egulow;       /* kleinster Mewert in physikalischer Einheit*/
   float eguhigh;      /* grter Mewert in physikalischer Einheit  */
   UINT32 datalength;  /* supercedes arraysize, when arraysize = -1  */
   char desc[4][64];   /* 4x Kommentarzeile                          */
} DHS;  /* Data Header Structure */
#define DHSStuctSize (32 + 4 + 2*2 + 4 + 4*4 + 16 + 4*2 + 4 + 4*64)

typedef struct
{
  char  deviceContext[CONTEXT_NAME_SIZE];
  char  deviceServerName[EXPORT_NAME_SIZE];
  char  deviceProperty[PROPERTY_NAME_SIZE];
  char  deviceName[DEVICE_NAME_SIZE];
  char  dataTag[TAG_NAME_SIZE];
  short dataFormat;
  short reserved;
  UINT32 dataIndex;
  UINT32 dataDimension;
} DEVSPEC; /* Device Specifier Release 4 */

typedef struct
{
  UINT32  eventNumber;
  UINT32  starttime;
  UINT32  stoptime;
  DEVSPEC devSpecifier;
} ADRB; /* Extended Archived Data Request Block */

EVNTOOLS_EXPORT int GetArchivedEventData(const char *context,const char *trigger,
                                     const char *devContext,const char *devServer,const char *devProperty,const char *devName,
                                     int dataIndex, int eventNumber, DTYPE *dout);
EVNTOOLS_EXPORT int GetArchivedEventList(const char *context,const char *trigger,time_t start,time_t stop,int *evnlst,int *lstsiz);
EVNTOOLS_EXPORT int GetArchivedPropertyInformation(const char *context,const char *fullname,PrpQueryStruct *pqs);
EVNTOOLS_EXPORT char *GetArchiveComment(const char *context,const char *trigger, int eventNumber);
EVNTOOLS_EXPORT int SendComment(const char *context,const char *trigger, int eventNumber,const char *cmt);
EVNTOOLS_EXPORT int GetArchiveTriggers(const char *context,NAME64 *triggers,int *triggerBufferSize);
EVNTOOLS_EXPORT int GetArchivedServers(const char *context,const char *trigger,NAME64 *srvlst,int *lstsiz);
EVNTOOLS_EXPORT int GetArchivedProperties(const char *context,const char *trigger,const char *devServer, NAME64 *prplst,int *lstsiz);

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* _EVENTTOOLS_H_ */
#endif /* _ARCEQM_H_ */