/** \file srvipx.h
 * \brief IPX transport structs and macros
 * \internal
 *
 */
#ifndef SRVIPX_H_
#define SRVIPX_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include "srvcore.h"
#ifdef IPXNW
# include <nwipxspx.h>
#endif


#define SPX_SRVSCK      0x45 /* listening SPX socket */
#define BRDCST_SOCKET   0x67 /* listening broadcast socket */
#define IPX_SRVSCK      0x55 /* listening registration socket */
#define MCAST_SOCKET    0x85 /* listening mcast socket */
#define NUM_LSN_ECB     2              /* allowed simultaneous connections */
#define NUM_RCV_ECB    (NUM_LSN_ECB*2) /* associated incoming contracts */
#define NUM_GLB_ECB     2              /* incoming broadcasts */
#define NUM_REG_ECB     2              /* incoming registration requests */
#define NUM_CLN_ECB     3              /* client RPC ecbs */
#define NUM_CNS_ECB     3              /* consumer ecbs */
#define NUM_MCST_ECB    2              /* incoming mcasts */

struct IPXNet
{
  BYTE network[4];
  BYTE node[6];
  BYTE ImmediateAddress[6];
  struct IPXNet *nextNet;
};
typedef struct
{
  BYTE net[4];  /* IPX Network Address */
  BYTE mac[6];  /* IPX Node (or MAC) Address */
  BYTE node[6]; /* IPX Immediate Address */
}
IPXNetListStruct;

extern WORD PCSocket;           /* IPX Producer/Consumer send socket */
extern struct IPXNet *IPXNets;
extern IPXNetListStruct *IPXNetList;
extern int NrOfIPXNets;
extern IPXNetListStruct *IPXBcastList;
extern int NrOfIPXBcastNets;
extern IPXAddress IPXFecAddress;
int IsMemberIPXNets(int num,IPXNetListStruct *lst,IPXAddress *ipxaddr);

extern int spxClnSck;   /**< SPX Client Socket */

#ifdef IPXNW

typedef struct ipxecb
{
  ECB       ecb;
  IPXHeader hdr;
  BYTE      buf[IPX_BUFFER_SIZE];
} IPXECB;
typedef struct spxecb
{
  ECB       ecb;
  SPXHeader hdr;
  BYTE      buf[IPX_DATA_SIZE];
} SPXECB;

extern SPXECB *clnECB[NUM_CLN_ECB];
extern SPXECB *conECB;
extern SPXECB *rcvECB[NUM_RCV_ECB];
extern SPXECB *lsnECB[NUM_LSN_ECB];
extern SPXECB *sndECB;

extern IPXECB *regECB[NUM_REG_ECB];
extern IPXECB *cnsECB[NUM_CNS_ECB];
extern IPXECB *glbECB[NUM_GLB_ECB];
extern IPXECB *brdcstECB;
extern IPXECB *mcstECB[NUM_MCST_ECB];
extern IPXECB *PCECB;

# define initSPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
    (e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(SPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_DATA_SIZE; }

# define initIPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
    (e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(IPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_BUFFER_SIZE; }

extern IPXECB *CreateIPXECB(int socket);
extern SPXECB *CreateSPXECB(int socket);

void clearStrandedPackets(SPXECB *ecbptr);
void clearStrandedConnection(SPXECB *ecbptr);

# define recvIncomingData(num,e,fcn) {\
    for (i=0; i<(num); i++) if (!(e)[i]->ecb.inUseFlag) {\
       if (!(e)[i]->ecb.completionCode) {\
         sprintf(gFecAddrBuffer,ipxnet_ntoa((e)[i]->hdr.source.node));\
         memcpy(&gIncomingIPXfec,&(e)[i]->hdr.source,sizeof(IPXAddress));\
         (fcn)(0,(e)[i]->buf);\
       }\
       IPXListenForPacket(&(e)[i]->ecb);\
     }\
   }

#else

extern int ipxMcastClnSck;
extern int ipxClnSck;
extern int ipxSrvSck;
extern int ipxGlbSck;
int ipxCreateSocket(UINT32 network, short port);

# define IPXECB void
# define SPXECB void

# define recvIncomingIPXData(sck) {\
    if (FD_ISSET(sck, &ipxconset)) {\
      n = recvfrom(sck,(char *)clnWorkArea,MIN_WORK_SIZE,0,\
          (struct sockaddr *)&IPXconsumer,&len);\
      if ( (n>0) && (n == PeekWord(*(UINT16 *)clnWorkArea))) {\
        memcpy(gIncomingIPXfec.node,IPXconsumer.sipx_node,6);\
        memcpy(gIncomingIPXfec.network,&IPXconsumer.sipx_network,4);\
        *(UINT16 *)gIncomingIPXfec.socket = ntohs(IPXconsumer.sipx_port);\
        memcpy(gIncomingIPXfec.socket,&IPXconsumer.sipx_port,2);\
        strcpy(gFecAddrBuffer,ipxnet_ntoa(gIncomingIPXfec.node));\
        consumeData(0,clnWorkArea);\
      }\
    }\
  }

#endif /* IPXNW */

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* SRVIPX_H_ */
